unit fAdvanced;

(*
  This sample is designed to show you some of the advanced features of XMLWorks,
  specifically :

  Interfaces :  If a published property is an Interface type, the streaming
    system will query for the IXMLWorksObject interface.

  TStrings : If a published property is of type TStrings (or descendant) the
    commatext will be used as the text value of the element.

  XMLString : By default all element text is encoded, for example "<" becomes
    "&gt;", the special "XMLString" type tells the system to bypass this step

  XMLMIMEString : If you want to transmit binary data over XML it must be
    encoded into a text format.  Declaring a public property of type
    XMLMIMEString will automatically handle this for you.

  XMLRGBTColor : TColor order ir BGR, however most users are used to editing
    colors using "RGB" in a hex format like "FF0000".  XMLRGBTColor allows you
    to stream colors in the user standard RGB hex format, yet the value is
    internally maintained as a standard TColor.  This is especially useful for
    storing GUI application settings.


  TDateTime : TDateTimes are encoded using the standard DateTimeToStr function.

  TTime : TTimes only stream the time portion

  TDate : TDates only stream the date portion

  Note : We plan on adding support for UTC in an upcoming release

  "Show XML" updates the Memo box with the current XML from the object.  "Read
    XML" parses the data from the memo back into the object.  You can experiment
    with changing the values in the memo box and seeing how the parser reads the
    values back.

  This also shows the two global booleans for controlling exceptions and XML
    element order.  Look at the "Unsorted XML" and "Error" code for
    explanations.
*)


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, XMLWorks2, ExtCtrls;

type

  ITester = interface
    procedure SetValue(const AValue: string);
  end;

  TTester = class(TXMLInterfacedObject, ITester)
  private
    fValue: String;
    procedure SetValue(const AValue: string);
  published
    property Value: String read fValue write fValue;
  end;

  TSampleObject = class(TXMLObject)
    private
      fMyInterface: ITester;
      fSimpleStrings: TStrings;
      fSimpleXMLString : XMLString;
      fSimpleString : String;
      fMyColor: XMLRGBTColor;

      fDateAndTime: TDateTime;
      fTimeOnly: TTime;
      fDateOnly: TDate;

      fBinaryData: String;
      fForm: TForm;

      function GetBinary: XMLMIMEString;
      procedure SetBinary(const Value: XMLMIMEString);
    public
      constructor Create(p_sXML: String = ''); override;

    published
      property MyInterface: ITester read fMyInterface write fMyInterface;
      property SimpleStrings: TStrings read fSimpleStrings write fSimpleStrings;
      property SimpleXMLString : XMLString read fSimpleXMLString write fSimpleXMLString;
      property SimpleString : String read fSimpleString write fSimpleString;
      property MyColor: XMLRGBTColor read fMyColor write fMyColor;

      property DateAndTime: TDateTime read fDateAndTime write fDateAndTime;
      property TimeOnly: TTime read fTimeOnly write fTimeOnly;
      property DateOnly: TDate read fDateOnly write fDateOnly;

      property ExampleBinary: XMLMIMEString read GetBinary write SetBinary;
      property InvalidXMLProperty: TForm read fForm write fForm;
    end;

  TfmAdvanced = class(TForm)
    memXML: TMemo;
    Panel1: TPanel;
    btnReadXML: TButton;
    btnShowXML: TButton;
    btnShowXMLUnsorted: TButton;
    btnGenError: TButton;

    procedure btnShowXMLClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnReadXMLClick(Sender: TObject);
    procedure btnShowXMLUnsortedClick(Sender: TObject);
    procedure btnGenErrorClick(Sender: TObject);
  private
    { Private declarations }
    fMyObject: TSampleObject;
  public
    { Public declarations }
  end;

var
  fmAdvanced: TfmAdvanced;


implementation

{$R *.DFM}

{ TTickerItem }

constructor TSampleObject.Create(p_sXML: String = '');
begin
  fMyInterface := TTester.Create;
  fSimpleStrings := TStringList.Create;

  inherited;
end;

function TSampleObject.GetBinary: XMLMIMEString;
begin
  result := fBinaryData;
end;

procedure TSampleObject.SetBinary(const Value: XMLMIMEString);
begin
  fBinaryData := Value;
end;

{ TTester }

procedure TTester.SetValue(const AValue: string);
begin
  fValue := AValue;
end;

{ TfmAdvanced }

procedure TfmAdvanced.FormCreate(Sender: TObject);
begin
  // Global Boolean causes an exception to be raised when generating XML for a class property that is not supported
  gb_XMLRaiseInvalidPropertyClass := False;

  fMyObject := TSampleObject.Create();
  with fMyObject do
  begin
    MyInterface.SetValue('TestX');
    SimpleStrings.Add('Line1');
    SimpleStrings.Add('Line2');
    SimpleStrings.Add('Line3');
    SimpleXMLString := '<TestMe></TestMe>';
    SimpleString := '<TestMe></TestMe>';

    MyColor := clRed;
    DateAndTime := Now;
    TimeOnly := Now;
    DateOnly := Now;
    ExampleBinary := 'This is only a sample, you can encode any type of data here';

    InvalidXMLProperty := self;
  end;
end;

procedure TfmAdvanced.btnShowXMLClick(Sender: TObject);
begin
  memXML.Text := fMyObject.XML;
end;

procedure TfmAdvanced.btnReadXMLClick(Sender: TObject);
begin
  fMyObject.XML := memXML.Text;
end;

procedure TfmAdvanced.btnShowXMLUnsortedClick(Sender: TObject);
begin
  // Global Boolean specifies the sort order of generated XML strings, False uses the declared order
  // true uses alphabetical order
  gb_XMLAlphaSortProperties := false;

  memXML.Text := fMyObject.XML;

  gb_XMLAlphaSortProperties := true;
end;

procedure TfmAdvanced.btnGenErrorClick(Sender: TObject);
begin
  // Global Boolean causes an exception to be raised when generating XML for a class property that is not supported
  gb_XMLRaiseInvalidPropertyClass := true;
  try
    memXML.Text := fMyObject.XML;
  finally
    gb_XMLRaiseInvalidPropertyClass := false;
  end;

end;

end.






