unit XMLObjectTestCases;


interface
(*
  XMLObjectTestCases
  Version   : 1.0
  Date      : 6/17/2002

  Author: Sancho Fock
          http://www.thsfock.de
          sancho@thsfock.de

  Description

    This Unit contains the Test Cases for TXMLObject of
    the Unit XMLWorks2
    *)

uses
  TestFramework, XMLWorks2;

type
 TXMLSimpleObject = class(TXMLObject)
 private
   FaString : string;
   FaInteger : integer;
   FaFloat : double;
   FaDate : TDateTime;
   FaBool : boolean;
 published
   property aString : string read FaString write FaString;
   property aInteger : integer read FaInteger write FaInteger;
   property aFloat : double read FaFloat write FaFloat;
   property aDate : TDateTime read FaDate write FaDate;
   property aBool : boolean read FaBool write FaBool;
 end;

 TXMLObjectTestCase = class(TTestCase)
 private
   FSourceObject : TXMLSimpleObject;
   FTargetObject : TXMLSimpleObject;
   procedure createSimpleObject;
   procedure createTargetObject;               overload;
   procedure createTargetObject(XML : string); overload;
   procedure assignSourcePropsToTarget;
 protected
   procedure setUp; override;
   procedure tearDown; override;
 public
 published
   procedure TestDTD;
   procedure TestDTDElements;
   procedure TestDTDSequence;
   procedure TestTagName;
   procedure TestStorage;
   procedure TestGetElementText;
   procedure TestGetXML;
 end;

const
  SI_VALUE     = 5;
  INT_VALUE    = 12345678;
  SINGLE_VALUE = 0.4;
  DOUBLE_VALUE = 12345678;
  CURR_VALUE   = 7489.90;
  DATE_VALUE   = 5000.200;
  STR_VALUE    = 'A Test string Value';
  BOOL_VALUE   = True;
  BYTE_VALUE   = 3;

implementation

uses
  sysutils;

{ TXMLObjectTestCase }

procedure TXMLObjectTestCase.assignSourcePropsToTarget;
begin
  FTargetObject.aString  := FSourceObject.aString ;
  FTargetObject.aInteger := FSourceObject.aInteger;
  FTargetObject.aFloat   := FSourceObject.aFloat;
  FTargetObject.aDate    := FSourceObject.aDate;
  FTargetObject.aBool    := FSourceObject.aBool;
end;

procedure TXMLObjectTestCase.createSimpleObject;
begin
  FSourceObject := TXMLSimpleObject.create;
  FSourceObject.aString  := STR_VALUE;
  FSourceObject.aInteger := INT_VALUE;
  FSourceObject.aFloat   := DOUBLE_VALUE;
  FSourceObject.aDate    := DATE_VALUE;
  FSourceObject.aBool    := BOOL_VALUE;
end;

procedure TXMLObjectTestCase.createTargetObject(XML: string);
begin
  FTargetObject := TXMLSimpleObject.create(XML);
end;

procedure TXMLObjectTestCase.createTargetObject;
begin
  FTargetObject := TXMLSimpleObject.create;
end;

procedure TXMLObjectTestCase.setUp;
begin
  inherited;
  self.createSimpleObject;
end;

procedure TXMLObjectTestCase.tearDown;
begin
  inherited;
  freeAndNil(FSourceObject);
  if assigned(FTargetObject) then
    freeAndNil(FTargetObject);
end;

procedure TXMLObjectTestCase.TestDTD;
begin
  self.createTargetObject; // create target for manually filling the XML
  FTargetObject.XML := FSourceObject.XML;
  try
    check(FSourceObject.GetDTD = FTargetObject.GetDTD,'man Assign : DTDs are different');
  finally
    FreeAndNil(FTargetObject);
  end;
  // part 2 filling up the Values
  self.createTargetObject; // create target for manually filling the properties
  self.assignSourcePropsToTarget;
  try
    check(FSourceObject.GetDTD = FTargetObject.GetDTD,'man Filling Prop : DTDs are different');
  finally
    FreeAndNil(FTargetObject);
  end;
  // part 3 filling up XML in the constructor
  self.createTargetObject(FSourceObject.XML); // create target for filling the properties in the constructor
  try
    check(FSourceObject.GetDTD = FTargetObject.GetDTD,'aut Assign : DTDs are different');
  finally
    FreeAndNil(FTargetObject);
  end;
end;

procedure TXMLObjectTestCase.TestDTDElements;
begin
  self.createTargetObject; // create target for manually filling the XML
  FTargetObject.XML := FSourceObject.XML;
  try
    check(FSourceObject.getDTDElements = FTargetObject.GetDTDElements,'man Assign : DTDElements are different');
  finally
    FreeAndNil(FTargetObject);
  end;
  // part 2 filling up the Values
  self.createTargetObject; // create target for manually filling the properties
  self.assignSourcePropsToTarget;
  try
    check(FSourceObject.GetDTDElements = FTargetObject.GetDTDElements,'man Filling Prop : DTDElements are different');
  finally
    FreeAndNil(FTargetObject);
  end;
  // part 3 filling up XML in the constructor
  self.createTargetObject(FSourceObject.XML); // create target for filling the properties in the constructor
  try
    check(FSourceObject.GetDTDElements = FTargetObject.GetDTDElements,'aut Assign : DTDElements are different');
  finally
    FreeAndNil(FTargetObject);
  end;
end;

procedure TXMLObjectTestCase.TestDTDSequence;
begin
  self.createTargetObject; // create target for manually filling the XML
  FTargetObject.XML := FSourceObject.XML;
  try
    check(FSourceObject.getDTDSequence = FTargetObject.GetDTDSequence,'man Assign : DTDSequence are different');
  finally
    FreeAndNil(FTargetObject);
  end;
  // part 2 filling up the Values
  self.createTargetObject; // create target for manually filling the properties
  self.assignSourcePropsToTarget;
  try
    check(FSourceObject.GetDTDSequence = FTargetObject.GetDTDSequence,'man Filling Prop : DTDSequence are different');
  finally
    FreeAndNil(FTargetObject);
  end;
  // part 3 filling up XML in the constructor
  self.createTargetObject(FSourceObject.XML); // create target for filling the properties in the constructor
  try
    check(FSourceObject.GetDTDSequence = FTargetObject.GetDTDSequence,'aut Assign : DTDSequence are different');
  finally
    FreeAndNil(FTargetObject);
  end;
end;

procedure TXMLObjectTestCase.TestGetElementText;
begin
  self.createTargetObject; // create target for manually filling the XML
  FTargetObject.XML := FSourceObject.XML;
  try
    check(FSourceObject.GetElementText = FTargetObject.getElementText,'man Assign : ElementText are different');
  finally
    FreeAndNil(FTargetObject);
  end;
  // part 2 filling up the Values
  self.createTargetObject; // create target for manually filling the properties
  self.assignSourcePropsToTarget;
  try
    check(FSourceObject.GetElementText = FTargetObject.GetElementText,'man Filling Prop : ElementText are different');
  finally
    FreeAndNil(FTargetObject);
  end;
  // part 3 filling up XML in the constructor
  self.createTargetObject(FSourceObject.XML); // create target for filling the properties in the constructor
  try
    check(FSourceObject.GetElementText = FTargetObject.GetElementText,'aut Assign : ElementText are different');
  finally
    FreeAndNil(FTargetObject);
  end;
end;

procedure TXMLObjectTestCase.TestGetXML;
begin
  self.createTargetObject; // create target for manually filling the XML
  FTargetObject.XML := FSourceObject.XML;
  try
    check(FSourceObject.XML = FTargetObject.XML,'man Assign : XML is different');
  finally
    FreeAndNil(FTargetObject);
  end;
  // part 2 filling up the Values
  self.createTargetObject; // create target for manually filling the properties
  self.assignSourcePropsToTarget;
  try
    check(FSourceObject.XML = FTargetObject.XML,'man Filling Prop : XML is different');
  finally
    FreeAndNil(FTargetObject);
  end;
  // part 3 filling up XML in the constructor
  self.createTargetObject(FSourceObject.XML); // create target for filling the properties in the constructor
  try
    check(FSourceObject.XML = FTargetObject.XML,'aut Assign : XML is different');
  finally
    FreeAndNil(FTargetObject);
  end;
end;

procedure TXMLObjectTestCase.TestStorage;
begin
  FSourceObject.SaveToFile(extractFilePath(ParamStr(0))+'unitTest.xml');
  self.createTargetObject;
  FTargetObject.LoadFromFile(extractFilePath(ParamStr(0))+'unitTest.xml');
  try
    check(FSourceObject.GetDTD = FTargetObject.GetDTD,'TestStorage : DTDs are different');
    check(FSourceObject.getDTDElements = FTargetObject.GetDTDElements,'TestStorage : DTDElements are different');
    check(FSourceObject.getDTDSequence = FTargetObject.GetDTDSequence,'TestStorage : DTDSequence are different');
    check(FSourceObject.GetElementText = FTargetObject.getElementText,'TestStorage : ElementText are different');
    check(FSourceObject.XML = FTargetObject.XML,'TestStorage : XML is different');
  finally
    FreeAndNil(FTargetObject);
  end;
end;

procedure TXMLObjectTestCase.TestTagName;
begin
  self.createTargetObject;
  try
    check(FSourceObject.getTagName = FTargetObject.GetTagName,'TagNames are different');
  finally
    FreeAndNil(FTargetObject);
  end;
end;

initialization
 TestFramework.RegisterTest(TXMLObjectTestCase.Suite);

end.
