unit fConfig;

(*

  This example is to demonstrate one of the main uses we have found for
    XMLWorks; storing configuration information.

  The xml generated in this sample will include ALL the supported published
    properties of TForm. For this sample the order of the elements is set to
    declaration order.  This means that our sample properties are listed at the
    bottom of the xml packet.  After running the app try clicking on "Show XML".
    Then go to the bottom of the memo box and edit any of the fields, then "Read
    XML".  You should see the related values change (i.e. FormCaption changes the
    caption of the Form, MemoColor is an RGB color for the background color of
    the memXML. DBConnection is included as an example of including "complex"
    configuration types.  Quite often at work we will create a custom
    "TAliasSettingsItem" (see XMLConfig.pas) with all the custom data we need
    for an application to be able to connect to the Dev, Test, and Prod instances.

  One of the really nice benefits of streaming settings like this is how quickly
    you can then add the ability to persist user settings.  For one suite of
    apps we were able to add the ability for users to persist settings to the DB
    with minimal code.  The system also allowed us to easily share global
    settings between similar applications, and to edit user settings via a
    simple web based interface (using XSL).
    
  In addition to persisting configuration information, XML streaming is a very
    efficient mechanism for querying the state of a server.  Using an Indy 
    HTTPServer on your server and XMLWorks you can very easily monitor and
    even control your server remotely using a simple web interface built with
    DHTML and XSL.
    
*)

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, XMLWorks2, XMLConfig, StdCtrls, ExtCtrls;

type
  TfrmConfig = class(TForm, IXMLWorksObject)
    Panel1: TPanel;
    btnShowXML: TButton;
    memXML: TMemo;
    btnReadXML: TButton;
    procedure FormCreate(Sender: TObject);
    procedure btnShowXMLClick(Sender: TObject);
    procedure btnReadXMLClick(Sender: TObject);
  private
    // Functions for the streaming system
    function GetElementText: string; virtual;
    function GetXML: string;
    procedure SetElementText(const p_sXML: string); virtual;
    procedure SetXML(const p_sXML: string);
    function getDTDElements: string;
    function getDTDSequence: string;
    class function getTagName: string; virtual;
    procedure LoadFromFile(FileName:string);
    procedure SaveToFile(FileName:string);
    property ElementText: string read GetElementText write SetElementText;
    property XML: string read GetXML write SetXML;
    function GetCaption: String;
    function GetMemoColor: XMLRGBTColor;
    procedure SetCaption(const Value: String);
    procedure SetMemoColor(const Value: XMLRGBTColor);


  public
    { Public declarations }
    fDBConnection : TServerConnectionSettings;
  published
    property DBConnection : TServerConnectionSettings read fDBConnection write fDBConnection;
    property MemoColor : XMLRGBTColor read GetMemoColor write SetMemoColor;
    property FormCaption : String read GetCaption write SetCaption;
  end;

var
  frmConfig: TfrmConfig;

implementation

{$R *.dfm}

{ TForm1 }

//*******  Streaming Functions ***************
(*

  You will notice that all of these functions are copy and past from
    TXMLInterfacedObject.  It is also possible to create the same type of
    functionality without copy and pasting using Interface proxies.  In addition
    all the user settings can be stored in a custom "TUserSettings" object that
    descends from a core XML object class.  I personally prefer a custom object,
    but this is to demonstrate how easily you can add XMLWorks streaming
    support to existing objects through the Interface.

*)

function TfrmConfig.getDTDElements: string;
begin
  result := getObjectDTDElements(self);
end;

function TfrmConfig.getDTDSequence: string;
begin
  result := getObjectDTDSequence(self);
end;

function TfrmConfig.GetElementText: string;
begin
  result := ObjectToXMLElements(self);
end;

class function TfrmConfig.getTagName: string;
begin
  result := 'ClientSettings';
end;

function TfrmConfig.GetXML: string;
begin
  result := '<' + getTagName + '>' + ElementText + '</' + getTagName + '>';
end;

procedure TfrmConfig.LoadFromFile(FileName: string);
begin
  XML := LoadStringFromFile(FileName);
end;

procedure TfrmConfig.SaveToFile(FileName: string);
begin
  SaveStringToFile(XML, FileName);
end;

procedure TfrmConfig.SetElementText(const p_sXML: string);
begin
  SetXMLObject(Self, p_sXML);
end;

procedure TfrmConfig.SetXML(const p_sXML: string);
begin
  ElementText := GetTagContent(p_sXML, getTagName);
end;

//*******  End Streaming Functions ***************


procedure TfrmConfig.FormCreate(Sender: TObject);
begin
  gb_XMLAlphaSortProperties := false;
  fDBConnection := TServerConnectionSettings.Create();
end;


function TfrmConfig.GetCaption: String;
begin
  result := Caption;
end;

function TfrmConfig.GetMemoColor: XMLRGBTColor;
begin
  result := memXML.Color;
end;

procedure TfrmConfig.SetCaption(const Value: String);
begin
  Caption := Value;
end;

procedure TfrmConfig.SetMemoColor(const Value: XMLRGBTColor);
begin
  memXML.Color := Value;
end;

procedure TfrmConfig.btnShowXMLClick(Sender: TObject);
begin
  memXML.Text := XML;
end;

procedure TfrmConfig.btnReadXMLClick(Sender: TObject);
begin
  XML := memXML.Text;
end;

end.
