unit XMLCollections;

interface

uses SysUtils, Classes, XMLWorks2;

type
  PIntList = ^TIntList;
  TIntList = array[0..MaxListSize - 1] of Integer;

  TXMLInterfaceCollectionItemClass = class of TXMLInterfaceCollectionItem;

  TXMLInterfaceCollection = class;
  TIntegerList = class (TObject)
  private
    FCapacity: Integer;
    FCount: Integer;
    FList: PIntList;
  protected
    function Get(Index: Integer): Integer;
    procedure Grow; virtual;
    procedure Put(Index: Integer; Item: Integer);
    procedure SetCapacity(NewCapacity: Integer);
    procedure SetCount(NewCount: Integer);
  public
    destructor Destroy; override;
    function Add(Item: Integer): Integer;
    procedure Clear; virtual;
    procedure Delete(Index: Integer);
    class procedure Error(Msg: string; Data: Integer); overload;
    procedure Exchange(Index1, Index2: Integer);
    function Extract(Item: Integer): Integer;
    function First: Integer;
    function IndexOf(Item: Integer): Integer;
    procedure Insert(Index: Integer; Item: Integer);
    function Last: Integer;
    procedure Move(CurIndex, NewIndex: Integer);
    function Remove(Item: Integer): Integer;
    property Capacity: Integer read FCapacity write SetCapacity;
    property Count: Integer read FCount write SetCount;
    property Items[Index: Integer]: Integer read Get write Put; default;
    property List: PIntList read FList;
  end;
  
  TXMLInterfaceCollectionItem = class (TPersistent, IUnknown, IXMLWorksObject)
  private
    FCollection: TXMLInterfaceCollection;
    FID: Integer;
    procedure SetCollection(Value: TXMLInterfaceCollection);
  protected
    FRefCount: Integer;
    procedure Changed(AllItems: Boolean);
    function GetDisplayName: string; virtual;
    function GetElementText: string; virtual;
    function GetIndex: Integer;
    function GetOwner: TPersistent; override;
    function GetXML: string;
    function getXMLAsProperties: string;
    function QueryInterface(const IID: TGUID; out Obj): HResult; stdcall;
    procedure SetDisplayName(const Value: string); virtual;
    procedure SetElementText(const p_sXML: string); virtual;
    procedure SetIndex(Value: Integer); virtual;
    procedure SetXML(const p_sXML: string);
    function _AddRef: Integer; stdcall;
    function _Release: Integer; stdcall;
  public
    constructor Create(Collection: TXMLInterfaceCollection); virtual;
    destructor Destroy; override;
    procedure AfterConstruction; override;
    procedure BeforeDestruction; override;
    function getDTDElements: string;
    function getDTDSequence: string;
    function GetNamePath: string; override;
    class function getTagName: string; virtual;
    procedure LoadFromFile(FileName:string);
    class function NewInstance: TObject; override;
    procedure SaveToFile(FileName:string);
    property Collection: TXMLInterfaceCollection read FCollection write 
            SetCollection;
    property DisplayName: string read GetDisplayName write SetDisplayName;
    property ElementText: string read GetElementText write SetElementText;
    property ID: Integer read FID;
    property Index: Integer read GetIndex write SetIndex;
    property RefCount: Integer read FRefCount;
    property XML: string read GetXML write SetXML;
  end;
  
  TXMLInterfaceCollection = class (TPersistent, IUnknown, IXMLWorksObject)
  private
    fIDList: TIntegerList;
    FItemClass: TXMLInterfaceCollectionItemClass;
    FItems: TList;
    FPropName: string;
    FUpdateCount: Integer;
    function GetCount: Integer;
    function GetPropName: string;
    procedure InsertItem(Item: TXMLInterfaceCollectionItem);
    procedure RemoveItem(Item: TXMLInterfaceCollectionItem);
  protected
    FRefCount: Integer;
    procedure Changed;
    function GetAttr(Index: Integer): string; dynamic;
    function GetAttrCount: Integer; dynamic;
    function GetElementText: string; virtual;
    function GetItem(Index: Integer): IUnknown;
    function GetItemAttr(Index, ItemIndex: Integer): string; dynamic;
    function GetXML: string;
    function QueryInterface(const IID: TGUID; out Obj): HResult; stdcall;
    procedure SetElementText(const p_sXML: string); virtual;
    procedure SetItem(Index: Integer; Value: IUnknown);
    procedure SetItemName(Item: TXMLInterfaceCollectionItem); virtual;
    procedure SetXML(const p_sXML: string);
    function _AddRef: Integer; stdcall;
    function _Release: Integer; stdcall;
    property PropName: string read GetPropName write FPropName;
    property UpdateCount: Integer read FUpdateCount;
  public
    constructor Create(ItemClass: TXMLInterfaceCollectionItemClass);
    destructor Destroy; override;
    function Add: IUnknown;
    procedure AfterConstruction; override;
    procedure Assign(Source: TPersistent); override;
    procedure BeforeDestruction; override;
    procedure BeginUpdate; virtual;
    procedure Clear;
    procedure Delete(Index: Integer);
    procedure EndUpdate; virtual;
    function FindItemID(ID: Integer): TXMLInterfaceCollectionItem;
    function getDTDElements: string;
    function getDTDSequence: string;
    function getItemTagName: string; virtual;
    function GetNamePath: string; override;
    function getPropertiesXML: string;
    function getSubsetElementText(Start, Ct: Integer): string;
    class function getTagName: string; virtual;
    function Insert(Index: Integer): IUnknown;
    procedure LoadFromFile(FileName:string);
    procedure Move(CurIndex, NewIndex: Integer);
    class function NewInstance: TObject; override;
    procedure SaveToFile(FileName:string);
    procedure Update(Item: TXMLInterfaceCollectionItem); virtual;
    property Count: Integer read GetCount;
    property ElementText: string read GetElementText write SetElementText;
    property ItemClass: TXMLInterfaceCollectionItemClass read FItemClass;
    property Items[Index: Integer]: IUnknown read GetItem write SetItem;
    property RefCount: Integer read FRefCount;
    property XML: string read GetXML write SetXML;
  end;
  
  TXMLOwnedInterfaceCollection = class (TXMLInterfaceCollection)
  private
    FOwner: TPersistent;
  protected
    function GetOwner: TPersistent; override;
  public
    constructor Create(AOwner: TPersistent; ItemClass: 
            TXMLInterfaceCollectionItemClass); reintroduce; virtual;
  end;
  
  TXMLOwnedCollection = class (TXMLCollection)
  private
    FOwner: TPersistent;
  protected
    function GetOwner: TPersistent; override;
  public
    constructor Create(AOwner: TPersistent; ItemClass: TXMLCollectionItemClass);
            reintroduce;
  end;
  


implementation
uses
  TypInfo;    // RTTI Stuff


{
********************************* TIntegerList *********************************
}
destructor TIntegerList.Destroy;
begin
  Clear;
end;

function TIntegerList.Add(Item: Integer): Integer;
begin
  Result := FCount;
  if Result = FCapacity then
    Grow;
  FList^[Result] := Item;
  Inc(FCount);
end;

procedure TIntegerList.Clear;
begin
  SetCount(0);
  SetCapacity(0);
end;

procedure TIntegerList.Delete(Index: Integer);
begin
  if (Index < 0) or (Index >= FCount) then
    Error('Index out of bounds (%d)', Index);
  Dec(FCount);
  if Index < FCount then
    System.Move(FList^[Index + 1], FList^[Index],
      (FCount - Index) * SizeOf(Pointer));
end;

class procedure TIntegerList.Error(Msg: string; Data: Integer);
begin
  raise Exception.CreateFmt(Msg, [Data]);
end;

procedure TIntegerList.Exchange(Index1, Index2: Integer);
var
  Item: Integer;
begin
  if (Index1 < 0) or (Index1 >= FCount) then
    Error('Index out of bounds (%d)', Index1);
  if (Index2 < 0) or (Index2 >= FCount) then
    Error('Index out of bounds (%d)', Index2);
  
  Item := FList^[Index1];
  FList^[Index1] := FList^[Index2];
  FList^[Index2] := Item;
end;

function TIntegerList.Extract(Item: Integer): Integer;
var
  I: Integer;
begin
  Result := -1;
  I := IndexOf(Item);
  if I >= 0 then
  begin
    Result := Item;
    Delete(I);
  end;
end;

function TIntegerList.First: Integer;
begin
  Result := Get(0);
end;

function TIntegerList.Get(Index: Integer): Integer;
begin
  if (Index < 0) or (Index >= FCount) then
    Error('Index out of bounds (%d)', Index);
  
  Result := FList^[Index];
end;

procedure TIntegerList.Grow;
var
  Delta: Integer;
begin
  if FCapacity > 64 then
    Delta := FCapacity div 4
  else
    if FCapacity > 8 then
      Delta := 16
    else
      Delta := 4;
  
  SetCapacity(FCapacity + Delta);
end;

function TIntegerList.IndexOf(Item: Integer): Integer;
begin
  Result := 0;
  
  while (Result < FCount) and (FList^[Result] <> Item) do
    Inc(Result);
  
  if Result = FCount then
    Result := -1;
end;

procedure TIntegerList.Insert(Index: Integer; Item: Integer);
begin
  if (Index < 0) or (Index > FCount) then
    Error('Index out of bounds (%d)', Index);
  
  if FCount = FCapacity then
    Grow;
  
  if Index < FCount then
    System.Move(FList^[Index], FList^[Index + 1],
      (FCount - Index) * SizeOf(Pointer));
  
  FList^[Index] := Item;
  
  Inc(FCount);
end;

function TIntegerList.Last: Integer;
begin
  Result := Get(FCount - 1);
end;

procedure TIntegerList.Move(CurIndex, NewIndex: Integer);
var
  Item: Integer;
begin
  if CurIndex <> NewIndex then
  begin
    if (NewIndex < 0) or (NewIndex >= FCount) then
      Error('Index out of bounds (%d)', NewIndex);
    Item := Get(CurIndex);
    Delete(CurIndex);
    Insert(NewIndex, Item);
  end;
end;

procedure TIntegerList.Put(Index: Integer; Item: Integer);
begin
  if (Index < 0) or (Index >= FCount) then
    Error('Index out of bounds (%d)', Index);
  
  FList^[Index] := Item;
end;

function TIntegerList.Remove(Item: Integer): Integer;
begin
  Result := IndexOf(Item);
  if Result >= 0 then
    Delete(Result);
end;

procedure TIntegerList.SetCapacity(NewCapacity: Integer);
begin
  if (NewCapacity < FCount) or (NewCapacity > MaxListSize) then
    Error('Invalid Capacity (%d)', NewCapacity);
  
  if NewCapacity <> FCapacity then
  begin
    ReallocMem(FList, NewCapacity * SizeOf(Pointer));
    FCapacity := NewCapacity;
  end;
end;

procedure TIntegerList.SetCount(NewCount: Integer);
var
  I: Integer;
begin
  if (NewCount < 0) or (NewCount > MaxListSize) then
    Error('Invalid NewCount (%d)', NewCount);
  
  if NewCount > FCapacity then
    SetCapacity(NewCount);
  
  if NewCount > FCount then
    FillChar(FList^[FCount], (NewCount - FCount) * SizeOf(Integer), 0)
  else
    for I := FCount - 1 downto NewCount do
      Delete(I);
  
  FCount := NewCount;
end;

{
************************* TXMLInterfaceCollectionItem **************************
}
constructor TXMLInterfaceCollectionItem.Create(Collection: 
        TXMLInterfaceCollection);
begin
  SetCollection(Collection);
end;

destructor TXMLInterfaceCollectionItem.Destroy;
begin
  SetCollection(nil);
  inherited Destroy;
end;

procedure TXMLInterfaceCollectionItem.AfterConstruction;
begin
  // Release the constructor's implicit refcount
  InterlockedDecrement(FRefCount);
end;

procedure TXMLInterfaceCollectionItem.BeforeDestruction;
begin
  if RefCount <> 0 then
    raise Exception.Create('Free must not be explicity called on an interfaced object');
end;

procedure TXMLInterfaceCollectionItem.Changed(AllItems: Boolean);
var
  Item: TXMLInterfaceCollectionItem;
begin
  if (FCollection <> nil) and (FCollection.FUpdateCount = 0) then
  begin
    if AllItems then
      Item := nil
    else
      Item := Self;
  
    FCollection.Update(Item);
  end;
end;

function TXMLInterfaceCollectionItem.GetDisplayName: string;
begin
  Result := ClassName;
end;

function TXMLInterfaceCollectionItem.getDTDElements: string;
begin
  result := getObjectDTDElements(self);
end;

function TXMLInterfaceCollectionItem.getDTDSequence: string;
begin
  result := getObjectDTDSequence(self);
end;

function TXMLInterfaceCollectionItem.GetElementText: string;
begin
  result := ObjectToXMLElements(self);
end;

function TXMLInterfaceCollectionItem.GetIndex: Integer;
begin
  if FCollection <> nil then
    Result := FCollection.FIDList[ID]
  else
    Result := -1;
end;

function TXMLInterfaceCollectionItem.GetNamePath: string;
begin
  if FCollection <> nil then
    Result := Format('%s[%d]',[FCollection.GetNamePath, Index])
  else
    Result := ClassName;
end;

function TXMLInterfaceCollectionItem.GetOwner: TPersistent;
begin
  Result := FCollection;
end;

class function TXMLInterfaceCollectionItem.getTagName: string;
begin
  result := Copy(ClassName, 2, Pos('Collection', ClassName) - 2);
end;

function TXMLInterfaceCollectionItem.GetXML: string;
begin
  result := '<' + getTagName + '>' + ElementText + '</' + getTagName + '>';
end;

function TXMLInterfaceCollectionItem.getXMLAsProperties: string;
begin
  result := '<' + getTagName + ' ' + ObjectToXMLProperties(self) + ' />'
end;

procedure TXMLInterfaceCollectionItem.LoadFromFile(FileName:string);
begin
  XML := LoadStringFromFile(FileName);
end;

class function TXMLInterfaceCollectionItem.NewInstance: TObject;
begin
  Result := inherited NewInstance;
  TXMLInterfaceCollectionItem(Result).FRefCount := 1;
end;

function TXMLInterfaceCollectionItem.QueryInterface(const IID: TGUID; out Obj): 
        HResult;
  
  const
    E_NOINTERFACE = HResult($80004002);
  
begin
  if GetInterface(IID, Obj) then Result := 0 else Result := E_NOINTERFACE;
end;

procedure TXMLInterfaceCollectionItem.SaveToFile(FileName:string);
begin
  SaveStringToFile(XML, FileName);
end;

procedure TXMLInterfaceCollectionItem.SetCollection(Value: 
        TXMLInterfaceCollection);
begin
  if FCollection <> Value then
  begin
    if FCollection <> nil then FCollection.RemoveItem(Self);
    if Value <> nil then Value.InsertItem(Self);
  end;
end;

procedure TXMLInterfaceCollectionItem.SetDisplayName(const Value: string);
begin
  Changed(False);
end;

procedure TXMLInterfaceCollectionItem.SetElementText(const p_sXML: string);
begin
  SetXMLObject(Self, p_sXML);
end;

procedure TXMLInterfaceCollectionItem.SetIndex(Value: Integer);
var
  CurIndex: Integer;
begin
  CurIndex := GetIndex;
  if (CurIndex >= 0) and (CurIndex <> Value) then
  begin
    FCollection.Move(CurIndex, Value);
    Changed(True);
  end;
end;

procedure TXMLInterfaceCollectionItem.SetXML(const p_sXML: string);
begin
  ElementText := GetTagContent(p_sXML, getTagName);
end;

function TXMLInterfaceCollectionItem._AddRef: Integer;
begin
  Result := InterlockedIncrement(FRefCount);
end;

function TXMLInterfaceCollectionItem._Release: Integer;
begin
  Result := InterlockedDecrement(FRefCount);
  if Result = 0 then
    Destroy;
end;

{
*************************** TXMLInterfaceCollection ****************************
}
constructor TXMLInterfaceCollection.Create(ItemClass: 
        TXMLInterfaceCollectionItemClass);
begin
  FItemClass := ItemClass;
  FItems := TList.Create;
  fIDList := TIntegerList.Create;
end;

destructor TXMLInterfaceCollection.Destroy;
begin
  FUpdateCount := 1;
  if FItems <> nil then
    Clear;
  FItems.Free;
  fIDList.Free;
  
  inherited Destroy;
end;

function TXMLInterfaceCollection.Add: IUnknown;
begin
  Result := FItemClass.Create(Self) as IUnknown;
end;

procedure TXMLInterfaceCollection.AfterConstruction;
begin
  // Release the constructor's implicit refcount
  InterlockedDecrement(FRefCount);
end;

procedure TXMLInterfaceCollection.Assign(Source: TPersistent);
begin
  {
  if Source is TXMLInterfaceCollectionItem then
  begin
    BeginUpdate;
    try
      Clear;
      for I := 0 to TCollection(Source).Count - 1 do
        TXMLInterfaceCollectionItem(Add).Assign(TXMLInterfaceCollectionItem(TXMLInterfaceCollection(Source).Items[I]));
    finally
      EndUpdate;
    end;
    Exit;
  end;
  }
  inherited Assign(Source);
end;

procedure TXMLInterfaceCollection.BeforeDestruction;
begin
  if RefCount <> 0 then
    raise Exception.Create('Free must not be explicity called on an interfaced object');
end;

procedure TXMLInterfaceCollection.BeginUpdate;
begin
  Inc(FUpdateCount);
end;

procedure TXMLInterfaceCollection.Changed;
begin
  if FUpdateCount = 0 then
    Update(nil);
end;

procedure TXMLInterfaceCollection.Clear;
begin
  if FItems.Count > 0 then
  begin
    BeginUpdate;
    try
      while FItems.Count > 0 do
        Delete(FItems.Count-1);
      fIDList.Clear;
    finally
      EndUpdate;
    end;
  end;
end;

procedure TXMLInterfaceCollection.Delete(Index: Integer);
begin
  TXMLInterfaceCollectionItem(FItems[Index]).Collection := nil;
end;

procedure TXMLInterfaceCollection.EndUpdate;
begin
  Dec(FUpdateCount);
  Changed;
end;

function TXMLInterfaceCollection.FindItemID(ID: Integer): 
        TXMLInterfaceCollectionItem;
begin
  result := fItems[fIDList[ID]];
end;

function TXMLInterfaceCollection.GetAttr(Index: Integer): string;
begin
  Result := '';
end;

function TXMLInterfaceCollection.GetAttrCount: Integer;
begin
  Result := 0;
end;

function TXMLInterfaceCollection.GetCount: Integer;
begin
  Result := FItems.Count;
end;

function TXMLInterfaceCollection.getDTDElements: string;
begin
  result := getObjectDTDElements(self);
end;

function TXMLInterfaceCollection.getDTDSequence: string;
begin
  result := getObjectDTDSequence(self);
  if result = '' then
    result := getItemTagName + '*'
  else
    result := getItemTagName + '*, ' + result;
end;

function TXMLInterfaceCollection.GetElementText: string;
var
  Index: Integer;
  StringList: TStringList;
  CollectionItemName: string;
begin
  result := '';
  StringList := TStringList.Create;
  try
    StringList.Text := getPropertiesXML;
    CollectionItemName := GetItemTagName;
    for Index := 0 to Count-1 do
    begin
      StringList.Add('<' + CollectionItemName + '>' + TXMLInterfaceCollectionItem(Items[Index]).ElementText + '</' + CollectionItemName + '>');
    end;
    result := StringList.Text;
  finally
    StringList.Free;
  end;
end;

function TXMLInterfaceCollection.GetItem(Index: Integer): IUnknown;
begin
  Result := TXMLInterfaceCollectionItem(FItems[Index]) as IUnknown;
end;

function TXMLInterfaceCollection.GetItemAttr(Index, ItemIndex: Integer): string;
begin
  Result := TXMLInterfaceCollectionItem(Items[ItemIndex]).DisplayName;
end;

function TXMLInterfaceCollection.getItemTagName: string;
begin
  result := ItemClass.getTagName;
end;

function TXMLInterfaceCollection.GetNamePath: string;
var
  S, P: string;
begin
  Result := ClassName;
  if GetOwner = nil then
    Exit;
  
  S := GetOwner.GetNamePath;
  
  if S = '' then
    Exit;
  
  P := PropName;
  if P = '' then
    Exit;
  
  Result := S + '.' + P;
end;

function TXMLInterfaceCollection.getPropertiesXML: string;
begin
  result := ObjectToXMLElements(self)
end;

function TXMLInterfaceCollection.GetPropName: string;
var
  I: Integer;
  Props: PPropList;
  TypeData: PTypeData;
  Owner: TPersistent;
begin
  Result := FPropName;
  Owner := GetOwner;
  if (Result <> '') or (Owner = nil) or (Owner.ClassInfo = nil) then Exit;
  TypeData := GetTypeData(Owner.ClassInfo);
  if (TypeData = nil) or (TypeData^.PropCount = 0) then Exit;
  GetMem(Props, TypeData^.PropCount * sizeof(Pointer));
  try
    GetPropInfos(Owner.ClassInfo, Props);
    for I := 0 to TypeData^.PropCount-1 do
    begin
      with Props^[I]^ do
        if (PropType^^.Kind = tkClass) and
          (GetOrdProp(Owner, Props^[I]) = Integer(Self)) then
          FPropName := Name;
    end;
  finally
    Freemem(Props);
  end;
  Result := FPropName;
end;

function TXMLInterfaceCollection.getSubsetElementText(Start, Ct: Integer): 
        string;
var
  StringList: TStringList;
  Index: Integer;
begin
  if (Start < 0) or (Start >= Count) then
    raise EXMLException.Create('Invalid Start (' + IntToStr(Start) + ')');
  
  if (Ct < 0) or ((Start + Ct) >= Count) then
    raise EXMLException.Create('Invalid Ct (' + IntToStr(Ct) + ')');
  
  result := '';
  StringList := TStringList.Create;
  try
    StringList.Text := ObjectToXMLElements(self);
    for Index := Start to Start+Ct-1 do
    begin
      StringList.Add(TXMLInterfaceCollectionItem(Items[Index]).XML);
    end;
    result := StringList.Text;
  finally
    StringList.Free;
  end;
end;

class function TXMLInterfaceCollection.getTagName: string;
begin
  result := Copy(ClassName, 2, Pos('Collection', ClassName) - 2) + '-list';
end;

function TXMLInterfaceCollection.GetXML: string;
begin
  result := '<' + getTagName + '>' + ElementText + '</' + getTagName + '>';
end;

function TXMLInterfaceCollection.Insert(Index: Integer): IUnknown;
begin
  Result := Add;
  TXMLInterfaceCollectionItem(Result).Index := Index;
end;

procedure TXMLInterfaceCollection.InsertItem(Item: TXMLInterfaceCollectionItem);
begin
  if not (Item is FItemClass) then
    raise EXMLInterfaceException.Create('Invalid Collection Item class - ' + Item.ClassName);
  
  // Force Ref Counting while in list
  Item._AddRef;
  Item.FCollection := Self;
  Item.FID := fIDList.Count;
  
  fIDList.Add(FItems.Add(Item));
end;

procedure TXMLInterfaceCollection.LoadFromFile(FileName:string);
begin
  XML := LoadStringFromFile(FileName);
end;

procedure TXMLInterfaceCollection.Move(CurIndex, NewIndex: Integer);
var
  I: Integer;
begin
  if CurIndex <> NewIndex then
  begin
    fItems.Move(CurIndex, NewIndex);
    if CurIndex > NewIndex then // Resort from New To Cur
      for i := NewIndex to CurIndex do
        fIDList[TXMLInterfaceCollectionItem(fItems[i]).fID] := i
    else
      for i := CurIndex to NewIndex do
        fIDList[TXMLInterfaceCollectionItem(fItems[i]).fID] := i
  end;
end;

class function TXMLInterfaceCollection.NewInstance: TObject;
begin
  Result := inherited NewInstance;
  TXMLInterfaceCollection(Result).FRefCount := 1;
end;

function TXMLInterfaceCollection.QueryInterface(const IID: TGUID; out Obj): 
        HResult;
  
  const
    E_NOINTERFACE = HResult($80004002);
  
begin
  if GetInterface(IID, Obj) then Result := 0 else Result := E_NOINTERFACE;
end;

procedure TXMLInterfaceCollection.RemoveItem(Item: TXMLInterfaceCollectionItem);
var
  X, OldID, OldIndex: Integer;
begin
  OldID := Item.fID;
  OldIndex := fIDList[OldID];
  
  
  for x := OldIndex to Count-1 do
    Dec(fIDList.FList^[TXMLInterfaceCollectionItem(fItems[X]).fID]);
  
  
  fIDList.Delete(OldID);
  FItems.Delete(OldIndex);
  
  for x := OldID to Count-1 do
    Dec(TXMLInterfaceCollectionItem(fItems[fIDList[X]]).fID);
  
  Item.FCollection := nil;
  Item._Release;
end;

procedure TXMLInterfaceCollection.SaveToFile(FileName:string);
begin
  SaveStringToFile(XML, FileName);
end;

procedure TXMLInterfaceCollection.SetElementText(const p_sXML: string);
var
  CurrentTagIndex, OverAllIndex: Integer;
  CollectionItemName, CurrentTag, CurrentTagContent: string;
begin
  Clear;
  
  CollectionItemName := GetItemTagName;
  
  CurrentTagIndex := 1;
  OverallIndex := 1;
  repeat
    CurrentTag := FastParseTag(p_sXML, '<' , '>', OverallIndex);
    CurrentTagContent := FastParseTagXML(p_sXML, CurrentTag, CurrentTagIndex);
  
    if (Length(CurrentTag) > 0) then
    begin
      if (CurrentTag = CollectionItemName) then
        TXMLInterfaceCollectionItem(Add).ElementText := CurrentTagContent
      else
        SetPropAsString(Self, GetPropInfo(ClassInfo, CurrentTag), CurrentTagContent);
    end;
  
    OverAllIndex := CurrentTagIndex;
  until (OverAllIndex<1) or (OverAllIndex > Length(p_sXML));
end;

procedure TXMLInterfaceCollection.SetItem(Index: Integer; Value: IUnknown);
begin
  //TXMLInterfaceCollectionItem(FItems[Index]).AssignIntf(Value);
  assert(false, 'Not implemented');
  { TODO : Do a query interface for some custom intf, then .Assign ... }
end;

procedure TXMLInterfaceCollection.SetItemName(Item: 
        TXMLInterfaceCollectionItem);
begin
end;

procedure TXMLInterfaceCollection.SetXML(const p_sXML: string);
begin
  ElementText := GetTagContent(p_sXML, getTagName);
end;

procedure TXMLInterfaceCollection.Update(Item: TXMLInterfaceCollectionItem);
begin
end;

function TXMLInterfaceCollection._AddRef: Integer;
begin
  Result := InterlockedIncrement(FRefCount);
end;

function TXMLInterfaceCollection._Release: Integer;
begin
  Result := InterlockedDecrement(FRefCount);
  if Result = 0 then
    Destroy;
end;

{
************************* TXMLOwnedInterfaceCollection *************************
}
constructor TXMLOwnedInterfaceCollection.Create(AOwner: TPersistent; ItemClass: 
        TXMLInterfaceCollectionItemClass);
begin
  FOwner := AOwner;
  inherited Create(ItemClass);
end;

function TXMLOwnedInterfaceCollection.GetOwner: TPersistent;
begin
  result := FOwner;
end;

{
***************************** TXMLOwnedCollection ******************************
}
constructor TXMLOwnedCollection.Create(AOwner: TPersistent; ItemClass: 
        TXMLCollectionItemClass);
begin
  FOwner := AOwner;
  inherited Create(ItemClass);
end;

function TXMLOwnedCollection.GetOwner: TPersistent;
begin
  result := FOwner;
end;


end.
